/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.util.jms;

import org.apache.wsif.WSIFCorrelationId;

/**
 * A WSIFCorrelationId is used to identify asynchronous 
 * requests so that a request can be matched to its response.
 *
 * @author Ant Elder <ant.elder@uk.ibm.com>
 */
public class WSIFJMSCorrelationId implements WSIFCorrelationId {
	private static final long serialVersionUID = 1L;

    private String id;

    /**
     * Creates a new WSIFJMSCorrelationId.
     * 
     * @param id   the id string
     */
    public WSIFJMSCorrelationId(String id) {
        if (id.length() < 256) {
            this.id = id;
        } else {
            throw new IllegalArgumentException("id length must be less than 256 characters");
        }
    }

    /**
     * gets the value of this WSIFJMSCorrelationId as a String.
     * 
     * @param the String value of this id
     */
    public String getCorrelationId() {
        return id;
    }

    /**
     * gets the value of this WSIFJMSCorrelationId as a byte array.
     * 
     * @param a byte array of this id
     */
    public byte[] getCorrelationIdAsBytes() {
        return id.getBytes();
    }

    /**
     * Compares this WSIFJMSCorrelationId to the specified object.
     * The result is true if and only if the argument is not null 
     * and is a WSIFCorrelationId object that represents the same
     * String id as this object.
     * 
     * @return true if the WSIFCorrelationId's have the same id,
     *         otherwise false;
     */
    public boolean equals(Object cid) {
        if (cid != null && cid instanceof WSIFCorrelationId) {
            return this.id.equals(((WSIFCorrelationId) cid).getCorrelationId());
        } else {
            return false;
        }
    }

    /**
     * The String representation of this object is returned.
     *
     * @return  the id string.
     */
    public String toString() {
        return id;
    }

    /**
     * Returns a hashcode for this object.
     *
     * @return  a hash code value for this object.
     */
    public int hashCode() {
        return id.hashCode();
    }
}