/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2002 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:  
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "WSIF" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 2001, 2002, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */

package org.apache.wsif.wsdl.extensions.jms;

import java.io.Serializable;

import javax.wsdl.extensions.ExtensibilityElement;
import javax.xml.namespace.QName;

/**
 * WSDL Jms service-port extension
 * 
 * @author <a href="mailto:ake@de.ibm.com">Hermann Akermann</a>
 * @author Ant Elder <antelder@apache.org>
 * @author Mark Whitlock <whitlock@apache.org>
 */
public class JMSAddress implements ExtensibilityElement, Serializable {

	private static final long serialVersionUID = 1L;

    protected QName fieldElementType = JMSConstants.Q_ELEM_JMS_ADDRESS;
    // Uses the wrapper type so we can tell if it was set or not.
    
    protected Boolean fieldRequired = null;

    protected String jmsVendorURI;
    protected String initCxtFact;
    protected String jndiProvURL;
    protected String destStyle;
    protected String jndiConnFactName;
    protected String jndiDestName;
    protected String jmsProvDestName;
    protected String jmsImplSpecURI;
    protected java.util.List propertyValues;

    /**
     * accessors
     */
    public String getJmsVendorURI() {
        return jmsVendorURI;
    }

    public String getInitCxtFact() {
        return initCxtFact;
    }

    public String getJndiProvURL() {
        return jndiProvURL;
    }

    public String getDestStyle() {
        return destStyle;
    }

    public String getJndiConnFactName() {
        return jndiConnFactName;
    }

    public String getJndiDestName() {
        return jndiDestName;
    }

    public String getJmsProvDestName() {
        return jmsProvDestName;
    }

    public String getJmsImplSpecURI() {
        return jmsImplSpecURI;
    }

    /**
     * mutators
     */
    public void setJmsVendorURI(String rhs) {
        jmsVendorURI = rhs;
    }

    public void setInitCxtFact(String rhs) {
        initCxtFact = rhs;
    }

    public void setJndiProvURL(String rhs) {
        jndiProvURL = rhs;
    }

    public void setDestStyle(String rhs) {
        destStyle = rhs;
    }

    public void setJndiConnFactName(String rhs) {
        jndiConnFactName = rhs;
    }

    public void setJndiDestName(String rhs) {
        jndiDestName = rhs;
    }

    public void setJmsProvDestName(String rhs) {
        jmsProvDestName = rhs;
    }

    public void setJmsImplSpecURI(String rhs) {
        jmsImplSpecURI = rhs;
    }

    /**
     * @see ExtensibilityElement#setElementType(QName)
     */
    public void setElementType(QName elementType) {
        fieldElementType = elementType;
    }

    /**
     * @see ExtensibilityElement#getElementType()
     */
    public QName getElementType() {
        return fieldElementType;
    }

    /**
     * @see ExtensibilityElement#setRequired(Boolean)
     */
    public void setRequired(Boolean required) {
        fieldRequired = required;
    }

    /**
     * @see ExtensibilityElement#getRequired()
     */
    public Boolean getRequired() {
        return fieldRequired;
    }

    public String toString() {
        StringBuffer strBuf = new StringBuffer(super.toString());

        strBuf.append("\nJmsAddress (" + fieldElementType + "):");
        strBuf.append("\nrequired=" + fieldRequired);

        strBuf.append("\njmsVendorURI=" + jmsVendorURI == null ? "null" : jmsVendorURI);
        strBuf.append("\ninitCxtFact=" + initCxtFact == null ? "null" : initCxtFact);
        strBuf.append("\njndiProvURL=" + jndiProvURL == null ? "null" : jndiProvURL);
        strBuf.append("\ndestStyle=" + destStyle == null ? "null" : destStyle);
        strBuf.append(
            "\njndiConnFactName=" + jndiConnFactName == null ? "null" : jndiConnFactName);
        strBuf.append("\njndiDestName=" + jndiDestName == null ? "null" : jndiDestName);
        strBuf.append(
            "\njmsProvDestName=" + jmsProvDestName == null ? "null" : jmsProvDestName);
        strBuf.append(
            "\njmsImplSpecURI=" + jmsImplSpecURI == null ? "null" : jmsImplSpecURI);

        return strBuf.toString();
    }

    public void addJMSPropertyValue(JMSPropertyValue jmsPropertyValue) {
        getJMSPropertyValues().add(jmsPropertyValue);
    }

    public java.util.List getJMSPropertyValues() {
        if (propertyValues == null)
            propertyValues = new java.util.ArrayList();
        return propertyValues;
    }
}